@@ -673,7 +673,7 @@ uri_normalized_copy (const char *part, int length,
 				*d++ = *s;
 			}
 		} else {
-			if (*s == ' ')
+			if (!g_ascii_isgraph (*s))
 				need_fixup = TRUE;
 			*d++ = *s;
 		}
@@ -681,16 +681,16 @@ uri_normalized_copy (const char *part, int length,
 
 	if (need_fixup) {
 		GString *fixed;
-		char *sp, *p;
 
 		fixed = g_string_new (NULL);
-		p = normalized;
-		while ((sp = strchr (p, ' '))) {
-			g_string_append_len (fixed, p, sp - p);
-			g_string_append (fixed, "%20");
-			p = sp + 1;
+		s = (guchar *)normalized;
+		while (*s) {
+			if (g_ascii_isgraph (*s))
+				g_string_append_c (fixed, *s);
+			else
+				g_string_append_printf (fixed, "%%%02X", (int)*s);
+			s++;
 		}
-		g_string_append (fixed, p);
 		g_free (normalized);
 		normalized = g_string_free (fixed, FALSE);
 	}
@@ -742,10 +742,6 @@ soup_uri_normalize (const char *part, const char *unescape_extra)
 gboolean
 soup_uri_uses_default_port (SoupURI *uri)
 {
-	g_return_val_if_fail (uri->scheme == SOUP_URI_SCHEME_HTTP ||
-			      uri->scheme == SOUP_URI_SCHEME_HTTPS ||
-			      uri->scheme == SOUP_URI_SCHEME_FTP, FALSE);
-
 	return uri->port == soup_scheme_default_port (uri->scheme);
 }
 
@@ -1050,11 +1046,11 @@ soup_uri_set_fragment (SoupURI *uri, const char *fragment)
 
 /**
  * soup_uri_copy_host:
- * @uri: a #SoupUri
+ * @uri: a #SoupURI
  *
  * Makes a copy of @uri, considering only the protocol, host, and port
  *
- * Return value: the new #SoupUri
+ * Return value: the new #SoupURI
  *
  * Since: 2.26.3
  **/
