@@ -1,4 +1,10 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+#include <config.h>
+
+#include <stdio.h>
+#include <string.h>
+#include <unistd.h>
+
+#include <libsoup/soup.h>
 
 #include "test-utils.h"
 
@@ -8,7 +14,6 @@ typedef struct {
 
 static struct RequestTest {
 	const char *description;
-	const char *bugref;
 	const char *request;
 	int length;
 	guint status;
@@ -20,14 +25,14 @@ static struct RequestTest {
 	/*** VALID REQUESTS ***/
 	/**********************/
 
-	{ "HTTP 1.0 request with no headers", NULL,
+	{ "HTTP 1.0 request with no headers",
 	  "GET / HTTP/1.0\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_0,
 	  { { NULL } }
 	},
 
-	{ "Req w/ 1 header", NULL,
+	{ "Req w/ 1 header",
 	  "GET / HTTP/1.1\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -36,7 +41,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 1 header, no leading whitespace", NULL,
+	{ "Req w/ 1 header, no leading whitespace",
 	  "GET / HTTP/1.1\r\nHost:example.com\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -45,7 +50,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 1 header including trailing whitespace", NULL,
+	{ "Req w/ 1 header including trailing whitespace",
 	  "GET / HTTP/1.1\r\nHost: example.com \r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -54,7 +59,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 1 header, wrapped", NULL,
+	{ "Req w/ 1 header, wrapped",
 	  "GET / HTTP/1.1\r\nFoo: bar\r\n baz\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -63,7 +68,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 1 header, wrapped with additional whitespace", NULL,
+	{ "Req w/ 1 header, wrapped with additional whitespace",
 	  "GET / HTTP/1.1\r\nFoo: bar \r\n  baz\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -72,7 +77,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 1 header, wrapped with tab", NULL,
+	{ "Req w/ 1 header, wrapped with tab",
 	  "GET / HTTP/1.1\r\nFoo: bar\r\n\tbaz\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -81,7 +86,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 1 header, wrapped before value", NULL,
+	{ "Req w/ 1 header, wrapped before value",
 	  "GET / HTTP/1.1\r\nFoo:\r\n bar baz\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -90,7 +95,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 1 header with empty value", NULL,
+	{ "Req w/ 1 header with empty value",
 	  "GET / HTTP/1.1\r\nHost:\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -99,7 +104,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 2 headers", NULL,
+	{ "Req w/ 2 headers",
 	  "GET / HTTP/1.1\r\nHost: example.com\r\nConnection: close\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -109,7 +114,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 3 headers", NULL,
+	{ "Req w/ 3 headers",
 	  "GET / HTTP/1.1\r\nHost: example.com\r\nConnection: close\r\nBlah: blah\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -120,7 +125,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 3 headers, 1st wrapped", NULL,
+	{ "Req w/ 3 headers, 1st wrapped",
 	  "GET / HTTP/1.1\r\nFoo: bar\r\n baz\r\nConnection: close\r\nBlah: blah\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -131,7 +136,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 3 headers, 2nd wrapped", NULL,
+	{ "Req w/ 3 headers, 2nd wrapped",
 	  "GET / HTTP/1.1\r\nConnection: close\r\nBlah: blah\r\nFoo: bar\r\n baz\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -142,7 +147,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ 3 headers, 3rd wrapped", NULL,
+	{ "Req w/ 3 headers, 3rd wrapped",
 	  "GET / HTTP/1.1\r\nConnection: close\r\nBlah: blah\r\nFoo: bar\r\n baz\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -153,7 +158,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ same header multiple times", NULL,
+	{ "Req w/ same header multiple times",
 	  "GET / HTTP/1.1\r\nFoo: bar\r\nFoo: baz\r\nFoo: quux\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -162,7 +167,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Connection header on HTTP/1.0 message", NULL,
+	{ "Connection header on HTTP/1.0 message",
 	  "GET / HTTP/1.0\r\nFoo: bar\r\nConnection: Bar, Quux\r\nBar: baz\r\nQuux: foo\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_0,
@@ -172,37 +177,13 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "GET with full URI", "667637",
-	  "GET http://example.com HTTP/1.1\r\n", -1,
-	  SOUP_STATUS_OK,
-	  "GET", "http://example.com", SOUP_HTTP_1_1,
-	  { { NULL } }
-	},
-
-	{ "GET with full URI in upper-case", "667637",
-	  "GET HTTP://example.com HTTP/1.1\r\n", -1,
-	  SOUP_STATUS_OK,
-	  "GET", "HTTP://example.com", SOUP_HTTP_1_1,
-	  { { NULL } }
-	},
-
-	/* It's better for this to be passed through: this means a SoupServer
-	 * could implement ftp-over-http proxying, for instance
-	 */
-	{ "GET with full URI of unrecognised scheme", "667637",
-	  "GET AbOuT: HTTP/1.1\r\n", -1,
-	  SOUP_STATUS_OK,
-	  "GET", "AbOuT:", SOUP_HTTP_1_1,
-	  { { NULL } }
-	},
-
 	/****************************/
 	/*** RECOVERABLE REQUESTS ***/
 	/****************************/
 
 	/* RFC 2616 section 4.1 says we SHOULD accept this */
 
-	{ "Spurious leading CRLF", NULL,
+	{ "Spurious leading CRLF",
 	  "\r\nGET / HTTP/1.1\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -213,7 +194,7 @@ static struct RequestTest {
 
 	/* RFC 2616 section 3.1 says we MUST accept this */
 
-	{ "HTTP/01.01 request", NULL,
+	{ "HTTP/01.01 request",
 	  "GET / HTTP/01.01\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -224,7 +205,7 @@ static struct RequestTest {
 
 	/* RFC 2616 section 19.3 says we SHOULD accept these */
 
-	{ "LF instead of CRLF after header", NULL,
+	{ "LF instead of CRLF after header",
 	  "GET / HTTP/1.1\r\nHost: example.com\nConnection: close\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -234,7 +215,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "LF instead of CRLF after Request-Line", NULL,
+	{ "LF instead of CRLF after Request-Line",
 	  "GET / HTTP/1.1\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -243,7 +224,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Mixed CRLF/LF", "666316",
+	{ "Mixed CRLF/LF",
 	  "GET / HTTP/1.1\r\na: b\r\nc: d\ne: f\r\ng: h\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -255,7 +236,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ incorrect whitespace in Request-Line", NULL,
+	{ "Req w/ incorrect whitespace in Request-Line",
 	  "GET  /\tHTTP/1.1\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -264,7 +245,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Req w/ incorrect whitespace after Request-Line", "475169",
+	{ "Req w/ incorrect whitespace after Request-Line",
 	  "GET / HTTP/1.1 \r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -275,9 +256,10 @@ static struct RequestTest {
 
 	/* If the request/status line is parseable, then we
 	 * just ignore any invalid-looking headers after that.
+	 * (qv bug 579318).
 	 */
 
-	{ "Req w/ mangled header", "579318",
+	{ "Req w/ mangled header",
 	  "GET / HTTP/1.1\r\nHost: example.com\r\nFoo one\r\nBar: two\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -287,7 +269,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "First header line is continuation", "666316",
+	{ "First header line is continuation",
 	  "GET / HTTP/1.1\r\n b\r\nHost: example.com\r\nc: d\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -297,7 +279,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Zero-length header name", "666316",
+	{ "Zero-length header name",
 	  "GET / HTTP/1.1\r\na: b\r\n: example.com\r\nc: d\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -307,7 +289,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "CR in header name", "666316",
+	{ "CR in header name",
 	  "GET / HTTP/1.1\r\na: b\r\na\rb: cd\r\nx\r: y\r\n\rz: w\r\nc: d\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -317,7 +299,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "CR in header value", "666316",
+	{ "CR in header value",
 	  "GET / HTTP/1.1\r\na: b\r\nHost: example\rcom\r\np: \rq\r\ns: t\r\r\nc: d\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -330,7 +312,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Tab in header name", "666316",
+	{ "Tab in header name",
 	  "GET / HTTP/1.1\r\na: b\r\na\tb: cd\r\nx\t: y\r\np: q\r\n\tz: w\r\nc: d\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -345,7 +327,7 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "Tab in header value", "666316",
+	{ "Tab in header value",
 	  "GET / HTTP/1.1\r\na: b\r\nab: c\td\r\nx: \ty\r\nz: w\t\r\nc: d\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_1,
@@ -358,92 +340,88 @@ static struct RequestTest {
 	  }
 	},
 
-	{ "NUL in header name", "760832",
-	  "GET / HTTP/1.1\r\nHost\x00: example.com\r\n", 36,
-	  SOUP_STATUS_OK,
-	  "GET", "/", SOUP_HTTP_1_1,
-	  { { "Host", "example.com" },
-	    { NULL }
-	  }
-	},
-
-	{ "NUL in header value", "760832",
-	  "GET / HTTP/1.1\r\nHost: example\x00" "com\r\n", 35,
-	  SOUP_STATUS_OK,
-	  "GET", "/", SOUP_HTTP_1_1,
-	  { { "Host", "examplecom" },
-	    { NULL }
-	  }
-	},
-
 	/************************/
 	/*** INVALID REQUESTS ***/
 	/************************/
 
-	{ "HTTP 0.9 request; not supported", NULL,
+	{ "HTTP 0.9 request; not supported",
 	  "GET /\r\n", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "HTTP 1.2 request (no such thing)", NULL,
+	{ "HTTP 1.2 request (no such thing)",
 	  "GET / HTTP/1.2\r\n", -1,
 	  SOUP_STATUS_HTTP_VERSION_NOT_SUPPORTED,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "HTTP 2000 request (no such thing)", NULL,
+	{ "HTTP 2000 request (no such thing)",
 	  "GET / HTTP/2000.0\r\n", -1,
 	  SOUP_STATUS_HTTP_VERSION_NOT_SUPPORTED,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "Non-HTTP request", NULL,
+	{ "Non-HTTP request",
 	  "GET / SOUP/1.1\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "Junk after Request-Line", NULL,
+	{ "Junk after Request-Line",
 	  "GET / HTTP/1.1 blah\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "NUL in Method", NULL,
+	{ "NUL in Method",
 	  "G\x00T / HTTP/1.1\r\nHost: example.com\r\n", 37,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "NUL at beginning of Method", "666316",
+	{ "NUL at beginning of Method",
 	  "\x00 / HTTP/1.1\r\nHost: example.com\r\n", 35,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "NUL in Path", NULL,
+	{ "NUL in Path",
 	  "GET /\x00 HTTP/1.1\r\nHost: example.com\r\n", 38,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "No terminating CRLF", NULL,
+	{ "NUL in header name",
+	  "GET / HTTP/1.1\r\n\x00: silly\r\n", 37,
+	  SOUP_STATUS_BAD_REQUEST,
+	  NULL, NULL, -1,
+	  { { NULL } }
+	},
+
+	{ "NUL in header value",
+	  "GET / HTTP/1.1\r\nHost: example\x00com\r\n", 37,
+	  SOUP_STATUS_BAD_REQUEST,
+	  NULL, NULL, -1,
+	  { { NULL } }
+	},
+
+	{ "No terminating CRLF",
 	  "GET / HTTP/1.1\r\nHost: example.com", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
 	  { { NULL } }
 	},
 
-	{ "Unrecognized expectation", NULL,
+	{ "Unrecognized expectation",
 	  "GET / HTTP/1.1\r\nHost: example.com\r\nExpect: the-impossible\r\n", -1,
 	  SOUP_STATUS_EXPECTATION_FAILED,
 	  NULL, NULL, -1,
@@ -454,37 +432,36 @@ static const int num_reqtests = G_N_ELEMENTS (reqtests);
 
 static struct ResponseTest {
 	const char *description;
-	const char *bugref;
 	const char *response;
 	int length;
 	SoupHTTPVersion version;
 	guint status_code;
 	const char *reason_phrase;
-	Header headers[10];
+	Header headers[4];
 } resptests[] = {
 	/***********************/
 	/*** VALID RESPONSES ***/
 	/***********************/
 
-	{ "HTTP 1.0 response w/ no headers", NULL,
+	{ "HTTP 1.0 response w/ no headers",
 	  "HTTP/1.0 200 ok\r\n", -1,
 	  SOUP_HTTP_1_0, SOUP_STATUS_OK, "ok",
 	  { { NULL } }
 	},
 
-	{ "HTTP 1.1 response w/ no headers", NULL,
+	{ "HTTP 1.1 response w/ no headers",
 	  "HTTP/1.1 200 ok\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { NULL } }
 	},
 
-	{ "Response w/ multi-word Reason-Phrase", NULL,
+	{ "Response w/ multi-word Reason-Phrase",
 	  "HTTP/1.1 400 bad request\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_BAD_REQUEST, "bad request",
 	  { { NULL } }
 	},
 
-	{ "Response w/ 1 header", NULL,
+	{ "Response w/ 1 header",
 	  "HTTP/1.1 200 ok\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
@@ -492,7 +469,7 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ 2 headers", NULL,
+	{ "Response w/ 2 headers",
 	  "HTTP/1.1 200 ok\r\nFoo: bar\r\nBaz: quux\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
@@ -501,7 +478,7 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ same header multiple times", NULL,
+	{ "Response w/ same header multiple times",
 	  "HTTP/1.1 200 ok\r\nFoo: bar\r\nFoo: baz\r\nFoo: quux\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar, baz, quux" },
@@ -509,7 +486,7 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ no reason phrase", NULL,
+	{ "Response w/ no reason phrase",
 	  "HTTP/1.1 200 \r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "",
 	  { { "Foo", "bar" },
@@ -517,15 +494,7 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ unknown status code", NULL,
-	  "HTTP/1.1 999 Request denied\r\nFoo: bar\r\n", -1,
-	  SOUP_HTTP_1_1, 999, "Request denied",
-	  { { "Foo", "bar" },
-	    { NULL }
-	  }
-	},
-
-	{ "Connection header on HTTP/1.0 message", NULL,
+	{ "Connection header on HTTP/1.0 message",
 	  "HTTP/1.0 200 ok\r\nFoo: bar\r\nConnection: Bar\r\nBar: quux\r\n", -1,
 	  SOUP_HTTP_1_0, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
@@ -534,26 +503,13 @@ static struct ResponseTest {
 	  }
 	},
 
-	/* Tests from Cockpit */
-
-	{ "Response w/ 3 headers, check case-insensitivity", "722341",
-	  "HTTP/1.0 200 ok\r\nHeader1: value3\r\nHeader2:  field\r\nHead3:  Another \r\n", -1,
-	  SOUP_HTTP_1_0, SOUP_STATUS_OK, "ok",
-	  { { "header1", "value3" },
-	    { "Header2", "field" },
-	    { "hEAD3", "Another" },
-	    { "Something else", NULL },
-	    { NULL }
-	  }
-	},
-
 	/*****************************/
 	/*** RECOVERABLE RESPONSES ***/
 	/*****************************/
 
 	/* RFC 2616 section 3.1 says we MUST accept this */
 
-	{ "HTTP/01.01 response", NULL,
+	{ "HTTP/01.01 response",
 	  "HTTP/01.01 200 ok\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
@@ -563,7 +519,7 @@ static struct ResponseTest {
 
 	/* RFC 2616 section 19.3 says we SHOULD accept these */
 
-	{ "Response w/ LF instead of CRLF after Status-Line", NULL,
+	{ "Response w/ LF instead of CRLF after Status-Line",
 	  "HTTP/1.1 200 ok\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
@@ -571,7 +527,7 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ incorrect spacing in Status-Line", NULL,
+	{ "Response w/ incorrect spacing in Status-Line",
 	  "HTTP/1.1  200\tok\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
@@ -579,7 +535,7 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ no reason phrase or preceding SP", NULL,
+	{ "Response w/ no reason phrase or preceding SP",
 	  "HTTP/1.1 200\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "",
 	  { { "Foo", "bar" },
@@ -587,7 +543,7 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ no whitespace after status code", NULL,
+	{ "Response w/ no whitespace after status code",
 	  "HTTP/1.1 200ok\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
@@ -596,7 +552,7 @@ static struct ResponseTest {
 	},
 
 	/* Shoutcast support */
-	{ "Shoutcast server not-quite-HTTP", "502325",
+	{ "Shoutcast server not-quite-HTTP",
 	  "ICY 200 OK\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_0, SOUP_STATUS_OK, "OK",
 	  { { "Foo", "bar" },
@@ -604,7 +560,8 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "Response w/ mangled header", "579318",
+	/* qv bug 579318, do_bad_header_tests() below */
+	{ "Response w/ mangled header",
 	  "HTTP/1.1 200 ok\r\nFoo: one\r\nBar two:2\r\nBaz: three\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "one" },
@@ -613,161 +570,95 @@ static struct ResponseTest {
 	  }
 	},
 
-	{ "HTTP 1.1 response with leading line break", "602863",
+	/* qv bug 602863 */
+	{ "HTTP 1.1 response with leading line break",
 	  "\nHTTP/1.1 200 ok\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
 	  { { "Foo", "bar" },
 	    { NULL } }
 	},
 
-	{ "NUL in header name", "760832",
-	  "HTTP/1.1 200 OK\r\nF\x00oo: bar\r\n", 28,
-	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "OK",
-	  { { "Foo", "bar" },
-	    { NULL }
-	  }
-	},
-
-	{ "NUL in header value", "760832",
-	  "HTTP/1.1 200 OK\r\nFoo: b\x00" "ar\r\n", 28,
-	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "OK",
-	  { { "Foo", "bar" },
-	    { NULL }
-	  }
-	},
-
-	/********************************/
-	/*** VALID CONTINUE RESPONSES ***/
-	/********************************/
-
-	/* Tests from Cockpit project */
-
-	{ "Response w/ 101 Switching Protocols + spaces after new line", NULL,
-	  "HTTP/1.0 101 Switching Protocols\r\n  \r\n", 38,
-	  SOUP_HTTP_1_0, SOUP_STATUS_SWITCHING_PROTOCOLS, "Switching Protocols",
-	  { { NULL } }
-	},
-
-	{ "Response w/ 101 Switching Protocols missing \\r + spaces", NULL,
-	  "HTTP/1.0  101  Switching Protocols\r\n  \r\n", 40,
-	  SOUP_HTTP_1_0, SOUP_STATUS_SWITCHING_PROTOCOLS, "Switching Protocols",
-	  { { NULL } }
-	},
-
-	{ "Response w/ 101 Switching Protocols + spaces after & before new line", NULL,
-	  "HTTP/1.1  101  Switching Protocols  \r\n  \r\n", 42,
-	  SOUP_HTTP_1_1, SOUP_STATUS_SWITCHING_PROTOCOLS, "Switching Protocols",
-	  { { NULL } }
-	},
-
 	/*************************/
 	/*** INVALID RESPONSES ***/
 	/*************************/
 
-	{ "Invalid HTTP version", NULL,
+	{ "Invalid HTTP version",
 	  "HTTP/1.2 200 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "Non-HTTP response", NULL,
+	{ "Non-HTTP response",
 	  "SOUP/1.1 200 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "Non-numeric status code", NULL,
+	{ "Non-numeric status code",
 	  "HTTP/1.1 XXX OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "No status code", NULL,
+	{ "No status code",
 	  "HTTP/1.1 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "One-digit status code", NULL,
+	{ "One-digit status code",
 	  "HTTP/1.1 2 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "Two-digit status code", NULL,
+	{ "Two-digit status code",
 	  "HTTP/1.1 20 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "Four-digit status code", NULL,
+	{ "Four-digit status code",
 	  "HTTP/1.1 2000 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "Status code < 100", NULL,
+	{ "Status code < 100",
 	  "HTTP/1.1 001 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "Status code > 999", NULL,
-	  "HTTP/1.1 1000 OK\r\nFoo: bar\r\n", -1,
+	{ "Status code > 599",
+	  "HTTP/1.1 600 OK\r\nFoo: bar\r\n", -1,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "NUL at start", "666316",
+	{ "NUL at start",
 	  "\x00HTTP/1.1 200 OK\r\nFoo: bar\r\n", 28,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "NUL in Reason Phrase", NULL,
+	{ "NUL in Reason Phrase",
 	  "HTTP/1.1 200 O\x00K\r\nFoo: bar\r\n", 28,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	/* Failing test from Cockpit */
-
-	{ "Partial response stops after HTTP/", NULL,
-	  "HTTP/", -1,
+	{ "NUL in header name",
+	  "HTTP/1.1 200 OK\r\nF\x00oo: bar\r\n", 28,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
 
-	{ "Space before HTTP/", NULL,
-	  " HTTP/1.0 101 Switching Protocols\r\n  ", -1,
+	{ "NUL in header value",
+	  "HTTP/1.1 200 OK\r\nFoo: b\x00ar\r\n", 28,
 	  -1, 0, NULL,
 	  { { NULL } }
 	},
-
-	{ "Missing reason", NULL,
-	  "HTTP/1.0  101\r\n  ", -1,
-	  -1, 0, NULL,
-	  { { NULL } }
-	},
-
-	{ "Response code containing alphabetic character", NULL,
-	  "HTTP/1.1  1A01  Switching Protocols  \r\n  ", -1,
-	  -1, 0, NULL,
-	  { { NULL } }
-	},
-
-	{ "TESTONE\\r\\n", NULL,
-	  "TESTONE\r\n  ", -1,
-	  -1, 0, NULL,
-	  { { NULL } }
-	},
-
-	{ "Response w/ 3 headers truncated", NULL,
-	  "HTTP/1.0 200 ok\r\nHeader1: value3\r\nHeader2:  field\r\nHead3:  Anothe", -1,
-	  -1, 0, NULL,
-	  { { NULL }
-	  }
-	},
 };
 static const int num_resptests = G_N_ELEMENTS (resptests);
 
@@ -795,11 +686,18 @@ static struct QValueTest {
 static const int num_qvaluetests = G_N_ELEMENTS (qvaluetests);
 
 static void
+print_header (const char *name, const char *value, gpointer data)
+{
+	debug_printf (1, "              '%s': '%s'\n", name, value);
+}
+
+static gboolean
 check_headers (Header *headers, SoupMessageHeaders *hdrs)
 {
 	GSList *header_names, *h;
 	SoupMessageHeadersIter iter;
 	const char *name, *value;
+	gboolean ok = TRUE;
 	int i;
 
 	header_names = NULL;
@@ -811,33 +709,36 @@ check_headers (Header *headers, SoupMessageHeaders *hdrs)
 	}
 
 	for (i = 0, h = header_names; headers[i].name && h; i++, h = h->next) {
-		g_assert (g_ascii_strcasecmp (h->data, headers[i].name) == 0);
-
+		if (strcmp (h->data, headers[i].name) != 0) {
+			ok = FALSE;
+			break;
+		}
 		value = soup_message_headers_get_list (hdrs, headers[i].name);
-		g_assert_cmpstr (value, ==, headers[i].value);
+		if (strcmp (value, headers[i].value) != 0) {
+			ok = FALSE;
+			break;
+		}
 	}
-	/* If we have remaining fields to check, they should return NULL */
-	for (; headers[i].name; i++) {
-		value = soup_message_headers_get_list (hdrs, headers[i].name);
-		g_assert_null (value);
-	}
-	g_assert_null (headers[i].name);
-	g_assert_null (h);
-
+	if (headers[i].name || h)
+		ok = FALSE;
 	g_slist_free (header_names);
+	return ok;
 }
 
 static void
 do_request_tests (void)
 {
-	int i, len;
+	int i, len, h;
 	char *method, *path;
 	SoupHTTPVersion version;
 	SoupMessageHeaders *headers;
 	guint status;
 
+	debug_printf (1, "Request tests\n");
 	for (i = 0; i < num_reqtests; i++) {
-		debug_printf (1, "%2d. %s (%s)\n", i + 1, reqtests[i].description,
+		gboolean ok = TRUE;
+
+		debug_printf (1, "%2d. %s (%s): ", i + 1, reqtests[i].description,
 			      soup_status_get_phrase (reqtests[i].status));
 
 		headers = soup_message_headers_new (SOUP_MESSAGE_HEADERS_REQUEST);
@@ -850,32 +751,71 @@ do_request_tests (void)
 		status = soup_headers_parse_request (reqtests[i].request, len,
 						     headers, &method, &path,
 						     &version);
-		g_assert_cmpint (status, ==, reqtests[i].status);
 		if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
-			g_assert_cmpstr (method, ==, reqtests[i].method);
-			g_assert_cmpstr (path, ==, reqtests[i].path);
-			g_assert_cmpint (version, ==, reqtests[i].version);
+			if ((reqtests[i].method && strcmp (reqtests[i].method, method) != 0) || !reqtests[i].method)
+				ok = FALSE;
+			if ((reqtests[i].path && strcmp (reqtests[i].path, path) != 0) || !reqtests[i].path)
+				ok = FALSE;
+			if (reqtests[i].version != version)
+				ok = FALSE;
 
-			check_headers (reqtests[i].headers, headers);
+			if (!check_headers (reqtests[i].headers, headers))
+				ok = FALSE;
+		} else {
+			if (status != reqtests[i].status)
+				ok = FALSE;
+		}
+
+		if (ok)
+			debug_printf (1, "OK!\n");
+		else {
+			debug_printf (1, "BAD!\n");
+			errors++;
+			if (reqtests[i].method) {
+				debug_printf (1, "    expected: '%s' '%s' 'HTTP/1.%d'\n",
+					      reqtests[i].method,
+					      reqtests[i].path,
+					      reqtests[i].version);
+				for (h = 0; reqtests[i].headers[h].name; h++) {
+					debug_printf (1, "              '%s': '%s'\n",
+						      reqtests[i].headers[h].name,
+						      reqtests[i].headers[h].value);
+				}
+			} else {
+				debug_printf (1, "    expected: %s\n",
+					      soup_status_get_phrase (reqtests[i].status));
+			}
+			if (method) {
+				debug_printf (1, "         got: '%s' '%s' 'HTTP/1.%d'\n",
+					      method, path, version);
+				soup_message_headers_foreach (headers, print_header, NULL);
+			} else {
+				debug_printf (1, "         got: %s\n",
+					      soup_status_get_phrase (status));
+			}
 		}
 
 		g_free (method);
 		g_free (path);
 		soup_message_headers_free (headers);
 	}
+	debug_printf (1, "\n");
 }
 
 static void
 do_response_tests (void)
 {
-	int i, len;
+	int i, len, h;
 	guint status_code;
 	char *reason_phrase;
 	SoupHTTPVersion version;
 	SoupMessageHeaders *headers;
 
+	debug_printf (1, "Response tests\n");
 	for (i = 0; i < num_resptests; i++) {
-		debug_printf (1, "%2d. %s (%s)\n", i + 1, resptests[i].description,
+		gboolean ok = TRUE;
+
+		debug_printf (1, "%2d. %s (%s): ", i + 1, resptests[i].description,
 			      resptests[i].reason_phrase ? "should parse" : "should NOT parse");
 
 		headers = soup_message_headers_new (SOUP_MESSAGE_HEADERS_RESPONSE);
@@ -888,17 +828,49 @@ do_response_tests (void)
 		if (soup_headers_parse_response (resptests[i].response, len,
 						 headers, &version,
 						 &status_code, &reason_phrase)) {
-			g_assert_cmpint (version, ==, resptests[i].version);
-			g_assert_cmpint (status_code, ==, resptests[i].status_code);
-			g_assert_cmpstr (reason_phrase, ==, resptests[i].reason_phrase);
+			if (resptests[i].version != version)
+				ok = FALSE;
+			if (resptests[i].status_code != status_code)
+				ok = FALSE;
+			if ((resptests[i].reason_phrase && strcmp (resptests[i].reason_phrase, reason_phrase) != 0) || !resptests[i].reason_phrase)
+				ok = FALSE;
 
-			check_headers (resptests[i].headers, headers);
-		} else
-			g_assert_null (resptests[i].reason_phrase);
+			if (!check_headers (resptests[i].headers, headers))
+				ok = FALSE;
+		} else {
+			if (resptests[i].reason_phrase)
+				ok = FALSE;
+		}
+
+		if (ok)
+			debug_printf (1, "OK!\n");
+		else {
+			debug_printf (1, "BAD!\n");
+			errors++;
+			if (resptests[i].reason_phrase) {
+				debug_printf (1, "    expected: 'HTTP/1.%d' '%03d' '%s'\n",
+					      resptests[i].version,
+					      resptests[i].status_code,
+					      resptests[i].reason_phrase);
+				for (h = 0; resptests[i].headers[h].name; h++) {
+					debug_printf (1, "              '%s': '%s'\n",
+						      resptests[i].headers[h].name,
+						      resptests[i].headers[h].value);
+				}
+			} else
+				debug_printf (1, "    expected: parse error\n");
+			if (reason_phrase) {
+				debug_printf (1, "         got: 'HTTP/1.%d' '%03d' '%s'\n",
+					      version, status_code, reason_phrase);
+				soup_message_headers_foreach (headers, print_header, NULL);
+			} else
+				debug_printf (1, "         got: parse error\n");
+		}
 
 		g_free (reason_phrase);
 		soup_message_headers_free (headers);
 	}
+	debug_printf (1, "\n");
 }
 
 static void
@@ -906,7 +878,9 @@ do_qvalue_tests (void)
 {
 	int i, j;
 	GSList *acceptable, *unacceptable, *iter;
+	gboolean wrong;
 
+	debug_printf (1, "qvalue tests\n");
 	for (i = 0; i < num_qvaluetests; i++) {
 		debug_printf (1, "%2d. %s:\n", i + 1, qvaluetests[i].header_value);
 
@@ -915,37 +889,48 @@ do_qvalue_tests (void)
 							     &unacceptable);
 
 		debug_printf (1, "    acceptable: ");
+		wrong = FALSE;
 		if (acceptable) {
-			/* Kludge to deal with the fact that the sort order of the first
-			 * test is not fully specified.
-			 */
-			if (i == 0 && acceptable->next &&
-			    !g_str_equal (acceptable->data, qvaluetests[i].acceptable[0]) &&
-			    g_str_equal (acceptable->data, qvaluetests[i].acceptable[1])) {
-				gpointer tmp = acceptable->data;
-				acceptable->data = acceptable->next->data;
-				acceptable->next->data = tmp;
-			}
-
 			for (iter = acceptable, j = 0; iter; iter = iter->next, j++) {
 				debug_printf (1, "%s ", (char *)iter->data);
-				g_assert_cmpstr (iter->data, ==, qvaluetests[i].acceptable[j]);
+				if (!qvaluetests[i].acceptable[j] ||
+				    strcmp (iter->data, qvaluetests[i].acceptable[j]) != 0)
+					wrong = TRUE;
 			}
 			debug_printf (1, "\n");
 			soup_header_free_list (acceptable);
 		} else
 			debug_printf (1, "(none)\n");
+		if (wrong) {
+			debug_printf (1, "    WRONG! expected: ");
+			for (j = 0; qvaluetests[i].acceptable[j]; j++)
+				debug_printf (1, "%s ", qvaluetests[i].acceptable[j]);
+			debug_printf (1, "\n");
+			errors++;
+		}
 
 		debug_printf (1, "  unacceptable: ");
+		wrong = FALSE;
 		if (unacceptable) {
 			for (iter = unacceptable, j = 0; iter; iter = iter->next, j++) {
 				debug_printf (1, "%s ", (char *)iter->data);
-				g_assert_cmpstr (iter->data, ==, qvaluetests[i].unacceptable[j]);
+				if (!qvaluetests[i].unacceptable[j] ||
+				    strcmp (iter->data, qvaluetests[i].unacceptable[j]) != 0)
+					wrong = TRUE;
 			}
 			debug_printf (1, "\n");
 			soup_header_free_list (unacceptable);
 		} else
 			debug_printf (1, "(none)\n");
+		if (wrong) {
+			debug_printf (1, "    WRONG! expected: ");
+			for (j = 0; qvaluetests[i].unacceptable[j]; j++)
+				debug_printf (1, "%s ", qvaluetests[i].unacceptable[j]);
+			debug_printf (1, "\n");
+			errors++;
+		}
+
+		debug_printf (1, "\n");
 	}
 }
 
@@ -969,6 +954,8 @@ do_content_disposition_tests (void)
 	SoupMultipart *multipart;
 	SoupMessageBody *body;
 
+	debug_printf (1, "Content-Disposition tests\n");
+
 	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
 	params = g_hash_table_new (g_str_hash, g_str_equal);
 	g_hash_table_insert (params, "filename", RFC5987_TEST_FILENAME);
@@ -976,7 +963,13 @@ do_content_disposition_tests (void)
 	g_hash_table_destroy (params);
 
 	header = soup_message_headers_get_one (hdrs, "Content-Disposition");
-	g_assert_cmpstr (header, ==, RFC5987_TEST_HEADER_ENCODED);
+	if (!strcmp (header, RFC5987_TEST_HEADER_ENCODED))
+		debug_printf (1, "  encoded OK\n");
+	else {
+		debug_printf (1, "  encoding FAILED!\n    expected: %s\n    got:      %s\n",
+			      RFC5987_TEST_HEADER_ENCODED, header);
+		errors++;
+	}
 
 	/* UTF-8 decoding */
 	soup_message_headers_clear (hdrs);
@@ -985,13 +978,22 @@ do_content_disposition_tests (void)
 	if (!soup_message_headers_get_content_disposition (hdrs,
 							   &disposition,
 							   &params)) {
-		soup_test_assert (FALSE, "UTF-8 decoding FAILED");
+		debug_printf (1, "  UTF-8 decoding FAILED!\n    could not parse\n");
+		errors++;
 		return;
 	}
 	g_free (disposition);
 
 	filename = g_hash_table_lookup (params, "filename");
-	g_assert_cmpstr (filename, ==, RFC5987_TEST_FILENAME);
+	if (!filename) {
+		debug_printf (1, "  UTF-8 decoding FAILED!\n    could not find filename\n");
+		errors++;
+	} else if (strcmp (filename, RFC5987_TEST_FILENAME) != 0) {
+		debug_printf (1, "  UTF-8 decoding FAILED!\n    expected: %s\n    got:      %s\n",
+			      RFC5987_TEST_FILENAME, filename);
+		errors++;
+	} else
+		debug_printf (1, "  UTF-8 decoded OK\n");
 	g_hash_table_destroy (params);
 
 	/* ISO-8859-1 decoding */
@@ -1001,13 +1003,22 @@ do_content_disposition_tests (void)
 	if (!soup_message_headers_get_content_disposition (hdrs,
 							   &disposition,
 							   &params)) {
-		soup_test_assert (FALSE, "iso-8859-1 decoding FAILED");
+		debug_printf (1, "  iso-8859-1 decoding FAILED!\n    could not parse\n");
+		errors++;
 		return;
 	}
 	g_free (disposition);
 
 	filename = g_hash_table_lookup (params, "filename");
-	g_assert_cmpstr (filename, ==, RFC5987_TEST_FILENAME);
+	if (!filename) {
+		debug_printf (1, "  iso-8859-1 decoding FAILED!\n    could not find filename\n");
+		errors++;
+	} else if (strcmp (filename, RFC5987_TEST_FILENAME) != 0) {
+		debug_printf (1, "  iso-8859-1 decoding FAILED!\n    expected: %s\n    got:      %s\n",
+			      RFC5987_TEST_FILENAME, filename);
+		errors++;
+	} else
+		debug_printf (1, "  iso-8859-1 decoded OK\n");
 	g_hash_table_destroy (params);
 
 	/* Fallback */
@@ -1017,19 +1028,27 @@ do_content_disposition_tests (void)
 	if (!soup_message_headers_get_content_disposition (hdrs,
 							   &disposition,
 							   &params)) {
-		soup_test_assert (FALSE, "fallback decoding FAILED");
+		debug_printf (1, "  fallback decoding FAILED!\n    could not parse\n");
+		errors++;
 		return;
 	}
 	g_free (disposition);
 
 	filename = g_hash_table_lookup (params, "filename");
-	g_assert_cmpstr (filename, ==, RFC5987_TEST_FALLBACK_FILENAME);
+	if (!filename) {
+		debug_printf (1, "  fallback decoding FAILED!\n    could not find filename\n");
+		errors++;
+	} else if (strcmp (filename, RFC5987_TEST_FALLBACK_FILENAME) != 0) {
+		debug_printf (1, "  fallback decoding FAILED!\n    expected: %s\n    got:      %s\n",
+			      RFC5987_TEST_FALLBACK_FILENAME, filename);
+		errors++;
+	} else
+		debug_printf (1, "  fallback decoded OK\n");
 	g_hash_table_destroy (params);
 
 	soup_message_headers_free (hdrs);
 
-	/* Ensure that soup-multipart always quotes filename */
-	g_test_bug ("641280");
+	/* Ensure that soup-multipart always quotes filename (bug 641280) */
 	multipart = soup_multipart_new (SOUP_FORM_MIME_TYPE_MULTIPART);
 	buffer = soup_buffer_new (SOUP_MEMORY_STATIC, "foo", 3);
 	soup_multipart_append_form_file (multipart, "test", "token",
@@ -1045,9 +1064,15 @@ do_content_disposition_tests (void)
 	buffer = soup_message_body_flatten (body);
 	soup_message_body_free (body);
 
-	g_assert_true (strstr (buffer->data, "filename=\"token\""));
-
+	if (strstr (buffer->data, "filename=\"token\""))
+		debug_printf (1, "  SoupMultipart encoded filename correctly\n");
+	else {
+		debug_printf (1, "  SoupMultipart encoded filename incorrectly!\n");
+		errors++;
+	}
 	soup_buffer_free (buffer);
+
+	debug_printf (1, "\n");
 }
 
 #define CONTENT_TYPE_TEST_MIME_TYPE "text/plain"
@@ -1064,7 +1089,7 @@ do_content_type_tests (void)
 	GHashTable *params;
 	const char *header, *mime_type;
 
-	g_test_bug ("576760");
+	debug_printf (1, "Content-Type tests\n");
 
 	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
 	params = g_hash_table_new (g_str_hash, g_str_equal);
@@ -1074,7 +1099,13 @@ do_content_type_tests (void)
 	g_hash_table_destroy (params);
 
 	header = soup_message_headers_get_one (hdrs, "Content-Type");
-	g_assert_cmpstr (header, ==, CONTENT_TYPE_TEST_HEADER);
+	if (!strcmp (header, CONTENT_TYPE_TEST_HEADER))
+		debug_printf (1, "  encoded OK\n");
+	else {
+		debug_printf (1, "  encoding FAILED!\n    expected: %s\n    got:      %s\n",
+			      CONTENT_TYPE_TEST_HEADER, header);
+		errors++;
+	}
 
 	soup_message_headers_clear (hdrs);
 	soup_message_headers_append (hdrs, "Content-Type",
@@ -1084,20 +1115,38 @@ do_content_type_tests (void)
 				     CONTENT_TYPE_TEST_MIME_TYPE);
 
 	mime_type = soup_message_headers_get_content_type (hdrs, &params);
-	g_assert_cmpstr (mime_type, ==, CONTENT_TYPE_TEST_MIME_TYPE);
-	g_assert_cmpint (g_hash_table_size (params), ==, 0);
+	if (!mime_type) {
+		debug_printf (1, "  decoding FAILED!\n    could not parse\n");
+		errors++;
+	}
+
+	if (mime_type && strcmp (mime_type, CONTENT_TYPE_TEST_MIME_TYPE) != 0) {
+		debug_printf (1, "  decoding FAILED!\n    bad returned MIME type: %s\n",
+			      mime_type);
+		errors++;
+	} else if (params && g_hash_table_size (params) != 0) {
+		debug_printf (1, "  decoding FAILED!\n    params contained %d params (should be 0)\n",
+			      g_hash_table_size (params));
+		errors++;
+	} else
+		debug_printf (1, "  decoded OK\n");
+
 	if (params)
 		g_hash_table_destroy (params);
 
-	g_test_bug ("577630");
-
 	soup_message_headers_clear (hdrs);
 	soup_message_headers_append (hdrs, "Content-Type",
 				     CONTENT_TYPE_BAD_HEADER);
 	mime_type = soup_message_headers_get_content_type (hdrs, &params);
-	g_assert_null (mime_type);
+	if (mime_type) {
+		debug_printf (1, "  Bad content rejection FAILED!\n");
+		errors++;
+	} else
+		debug_printf (1, "  Bad content rejection OK\n");
 
 	soup_message_headers_free (hdrs);
+
+	debug_printf (1, "\n");
 }
 
 struct {
@@ -1118,7 +1167,7 @@ do_append_param_tests (void)
 	GString *params;
 	int i;
 
-	g_test_bug ("577728");
+	debug_printf (1, "soup_header_g_string_append_param() tests\n");
 
 	params = g_string_new (NULL);
 	for (i = 0; i < G_N_ELEMENTS (test_params); i++) {
@@ -1128,8 +1177,15 @@ do_append_param_tests (void)
 						   test_params[i].name,
 						   test_params[i].value);
 	}
-	g_assert_cmpstr (params->str, ==, TEST_PARAMS_RESULT);
+	if (strcmp (params->str, TEST_PARAMS_RESULT) != 0) {
+		debug_printf (1, "  FAILED!\n    expected: %s\n    got: %s\n",
+			      TEST_PARAMS_RESULT, params->str);
+		errors++;
+	} else
+		debug_printf (1, "  OK\n");
 	g_string_free (params, TRUE);
+
+	debug_printf (1, "\n");
 }
 
 static const struct {
@@ -1152,15 +1208,19 @@ do_bad_header_tests (void)
 	SoupMessageHeaders *hdrs;
 	int i;
 
+	debug_printf (1, "bad header rejection tests\n");
+
 	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
 	for (i = 0; i < G_N_ELEMENTS (bad_headers); i++) {
 		debug_printf (1, "  %s\n", bad_headers[i].description);
-
-		g_test_expect_message ("libsoup", G_LOG_LEVEL_CRITICAL,
-				       "*soup_message_headers_append*assertion*failed*");
+		expect_warning = TRUE;
 		soup_message_headers_append (hdrs, bad_headers[i].name,
 					     bad_headers[i].value);
-		g_test_assert_expected_messages ();
+		if (expect_warning) {
+			expect_warning = FALSE;
+			debug_printf (1, "    FAILED: soup_message_headers_append() did not reject it\n");
+			errors++;
+		}
 	}
 	soup_message_headers_free (hdrs);
 }
@@ -1168,20 +1228,16 @@ do_bad_header_tests (void)
 int
 main (int argc, char **argv)
 {
-	int ret;
-
 	test_init (argc, argv, NULL);
 
-	g_test_add_func ("/header-parsing/request", do_request_tests);
-	g_test_add_func ("/header-parsing/response", do_response_tests);
-	g_test_add_func ("/header-parsing/qvalue", do_qvalue_tests);
-	g_test_add_func ("/header-parsing/content-disposition", do_content_disposition_tests);
-	g_test_add_func ("/header-parsing/content-type", do_content_type_tests);
-	g_test_add_func ("/header-parsing/append-param", do_append_param_tests);
-	g_test_add_func ("/header-parsing/bad", do_bad_header_tests);
-
-	ret = g_test_run ();
+	do_request_tests ();
+	do_response_tests ();
+	do_qvalue_tests ();
+	do_content_disposition_tests ();
+	do_content_type_tests ();
+	do_append_param_tests ();
+	do_bad_header_tests ();
 
 	test_cleanup ();
-	return ret;
+	return errors != 0;
 }
